@@ -1,10 +1,15 @@
 #![crate_name = "actix_extract_multipart"]
 
-use actix_multipart::Multipart;
+use actix_multipart;
 use futures::{StreamExt, TryStreamExt};
 use serde::{Deserialize};
 use serde_json::{Value, Number, Map};
 use std::str;
+use std::ops::{Deref, DerefMut};
+
+use actix_web::{dev::Payload, Error, FromRequest, HttpRequest};
+use futures_util::future::{Future};
+use std::pin::Pin;
 
 pub type FileData = Vec<u8>;
 
@@ -12,7 +17,6 @@ pub type FileData = Vec<u8>;
 pub struct File {
     file_type: String,
     name: String,
-    size: u64,
     data: FileData,
 }
 impl File {
@@ -22,22 +26,67 @@ impl File {
     pub fn name(&self) -> &String {
         &self.name
     }
-    pub fn len(&self) -> u64 {
-        self.size
+    pub fn len(&self) -> usize {
+        self.data.len()
     }
     pub fn data(&self) -> &FileData {
         &self.data
     }
 }
 
-pub async fn extract_multipart<T>(mut payload: Multipart) -> Result<T, ()>
-    where T: serde::de::DeserializeOwned
+fn params_insert(params: &mut Map<String, Value>, field_name: &str, field_name_formatted: &String, element: Value) {
+    if params.contains_key(&field_name_formatted.to_owned()) {
+        match params.get_mut(&field_name_formatted.to_owned()).unwrap() {
+            Value::Array(val) => {
+                val.push(element);
+            }
+            _ => ()
+        }
+    } else {
+        if field_name.ends_with("[]") {
+            params.insert(field_name_formatted.to_owned(), Value::Array(vec![element]));
+        } else {
+            params.insert(field_name_formatted.to_owned(), element);
+        }
+    }
+}
+
+pub struct Multipart<T> {
+    data: T
+}
+
+impl<T> Multipart<T> {
+    fn new(data: T) -> Self {
+        Multipart::<T> {
+            data
+        }
+    }
+}
+
+impl<T> Deref for Multipart<T> {
+    type Target = T;
+
+    fn deref(&self) -> &T {
+        &self.data
+    }
+}
+
+impl<T> DerefMut for Multipart<T> {
+    fn deref_mut(&mut self) -> &mut T {
+        &mut self.data
+    }
+}
+
+async fn extract_multipart<T>(mut payload: actix_multipart::Multipart) -> Result<T, serde_json::Error>
+where T: serde::de::DeserializeOwned
 {
     let mut params = Map::new();
 
     'mainWhile: while let Ok(Some(mut field)) = payload.try_next().await {
         if let Some(content_disposition) = field.content_disposition() {
             if let Some(field_name) = content_disposition.get_name() {
+                let field_name_formatted = field_name.replace("[]", "");
+
                 if let Some(file_name) = content_disposition.get_filename() {
                     let mut data: Vec<Value> = Vec::new();
 
@@ -51,73 +100,73 @@ pub async fn extract_multipart<T>(mut payload: Multipart) -> Result<T, ()>
                                 }
                             },
                             Err(_) => {
-                                params.insert(field_name.to_owned(), Value::Null);
+                                params.insert(field_name_formatted.to_owned(), Value::Null);
                                 continue 'mainWhile;
                             }
                         }
                     }
             
-                    let size: usize = data.len();
-
-                    if size == 0 {
+                    if data.len() == 0 {
                         continue 'mainWhile;
                     }
 
-                    // let file_type_str: String = get_file_type(field.content_type());
                     let file_type_str: String = field.content_type().to_string();
 
                     let mut sub_params = Map::new();
                     sub_params.insert("file_type".to_owned(), Value::String(file_type_str.clone()));
                     sub_params.insert("name".to_owned(), Value::String(file_name.to_string()));
-                    sub_params.insert("size".to_owned(), Value::Number(Number::from(size)));
                     sub_params.insert("data".to_owned(), Value::Array(data));
 
-                    if params.contains_key(&field_name.to_owned()) {
-                        match params.get_mut(&field_name.to_owned()).unwrap() {
-                            Value::Array(val) => {
-                                val.push(Value::Object(sub_params));
-                            }
-                            _ => {
-                                let pre_val = params.get(&field_name.to_owned()).unwrap().clone();
-                                params.remove(&field_name.to_owned());
-                                params.insert(field_name.to_owned(), Value::Array(vec![pre_val, Value::Object(sub_params)]));
-                            }
-                        }
-                    } else {
-                        params.insert(field_name.to_owned(), Value::Object(sub_params));
-                    }
+                    params_insert(&mut params, field_name, &field_name_formatted, Value::Object(sub_params));
                 } else {
                     if let Some(value) = field.next().await {
                         match value {
                             Ok(val) => match str::from_utf8(&val) {
                                 Ok(convert_str) => match convert_str.parse::<isize>() {
-                                    Ok(number) => params.insert(field_name.to_owned(), Value::Number(Number::from(number))),
+                                    Ok(number) => params_insert(&mut params, field_name, &field_name_formatted, Value::Number(Number::from(number))),
                                     Err(_) => match convert_str {
-                                        "true" => params.insert(field_name.to_owned(), Value::Bool(true)),
-                                        "false" => params.insert(field_name.to_owned(), Value::Bool(false)),
-                                        _ => params.insert(field_name.to_owned(), Value::String(convert_str.to_owned()))
+                                        "true" => params_insert(&mut params, field_name, &field_name_formatted, Value::Bool(true)),
+                                        "false" => params_insert(&mut params, field_name, &field_name_formatted, Value::Bool(false)),
+                                        _ => params_insert(&mut params, field_name, &field_name_formatted, Value::String(convert_str.to_owned()))
                                     },
                                 },
-                                Err(_) => params.insert(field_name.to_owned(), Value::Null)
+                                Err(_) => params_insert(&mut params, field_name, &field_name_formatted, Value::Null)
                             },
-                            Err(_) => params.insert(field_name.to_owned(), Value::Null),
+                            Err(_) => params_insert(&mut params, field_name, &field_name_formatted, Value::Null)
                         };
                     }
                 }
             }
         }
     }
-    
+
     match serde_json::from_value::<T>(Value::Object(params)) {
         Ok(final_struct) => Ok(final_struct),
-        Err(_) => Err(())
+        Err(e) => Err(e)
+    }
+}
+
+impl<T: serde::de::DeserializeOwned> FromRequest for Multipart<T> {
+    type Error = Error;
+    type Future = Pin<Box<dyn Future<Output = Result<Self, Self::Error>>>>;
+    type Config = ();
+
+    fn from_request(req: &HttpRequest, payload: &mut Payload) -> Self::Future {
+        let multipart = actix_multipart::Multipart::new(req.headers(), payload.take());
+        
+        Box::pin(async move {
+            match extract_multipart::<T>(multipart).await {
+                Ok(response) => Ok(Multipart::<T>::new(response)),
+                Err(_) => Err(actix_web::error::ErrorBadRequest(format!("The data received does not correspond to those expected")))
+            }
+        })
     }
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
-    use actix_multipart::Multipart;
+    use actix_multipart;
     use actix_web::http::header::{self, HeaderMap};
     use tokio::sync::mpsc;
     use actix_web::error::{PayloadError};
@@ -164,6 +213,114 @@ mod tests {
         );
         (bytes, headers)
     }
+    fn create_simple_request_with_array_header() -> (Bytes, HeaderMap) {
+        let bytes = Bytes::from(
+            "testasdadsad\r\n\
+             --abbc761f78ff4d7cb7573b5a23f96ef0\r\n\
+             Content-Disposition: form-data; name=\"u32_param[]\"\r\n\r\n\
+             56\r\n\
+             --abbc761f78ff4d7cb7573b5a23f96ef0\r\n\
+             Content-Disposition: form-data; name=\"u32_param[]\"\r\n\r\n\
+             49\r\n\
+             --abbc761f78ff4d7cb7573b5a23f96ef0\r\n\
+             Content-Disposition: form-data; name=\"i32_param[]\"\r\n\r\n\
+             -12\r\n\
+             --abbc761f78ff4d7cb7573b5a23f96ef0\r\n\
+             Content-Disposition: form-data; name=\"i32_param[]\"\r\n\r\n\
+             -2\r\n\
+             --abbc761f78ff4d7cb7573b5a23f96ef0\r\n\
+             Content-Disposition: form-data; name=\"i32_param[]\"\r\n\r\n\
+             -17\r\n\
+             --abbc761f78ff4d7cb7573b5a23f96ef0\r\n\
+             Content-Disposition: form-data; name=\"string_param[]\"\r\n\r\n\
+             A simple test\r\n\
+             --abbc761f78ff4d7cb7573b5a23f96ef0\r\n\
+             Content-Disposition: form-data; name=\"string_param[]\"\r\n\r\n\
+             A simple test2\r\n\
+             --abbc761f78ff4d7cb7573b5a23f96ef0\r\n\
+             Content-Disposition: form-data; name=\"string_param[]\"\r\n\r\n\
+             A simple test3\r\n\
+             --abbc761f78ff4d7cb7573b5a23f96ef0--\r\n",
+        );
+        let mut headers = HeaderMap::new();
+        headers.insert(
+            header::CONTENT_TYPE,
+            header::HeaderValue::from_static(
+                "multipart/mixed; boundary=\"abbc761f78ff4d7cb7573b5a23f96ef0\"",
+            ),
+        );
+        (bytes, headers)
+    }
+    fn create_simple_request_with_3_files_array_header() -> (Bytes, HeaderMap) {
+        let bytes = Bytes::from(
+            "testasdadsad\r\n\
+             --abbc761f78ff4d7cb7573b5a23f96ef0\r\n\
+             Content-Disposition: form-data; name=\"files_param[]\"; filename=\"fn.txt\"\r\n\
+             Content-Type: text/plain; charset=utf-8\r\n\r\n\
+             test\r\n\
+             --abbc761f78ff4d7cb7573b5a23f96ef0\r\n\
+             Content-Disposition: form-data; name=\"files_param[]\"; filename=\"fn2.txt\"\r\n\
+             Content-Type: text/plain; charset=utf-8\r\n\r\n\
+             test\r\n\
+             --abbc761f78ff4d7cb7573b5a23f96ef0\r\n\
+             Content-Disposition: form-data; name=\"files_param[]\"; filename=\"fn3.txt\"\r\n\
+             Content-Type: text/plain; charset=utf-8\r\n\r\n\
+             test\r\n\
+             --abbc761f78ff4d7cb7573b5a23f96ef0--\r\n",
+        );
+        let mut headers = HeaderMap::new();
+        headers.insert(
+            header::CONTENT_TYPE,
+            header::HeaderValue::from_static(
+                "multipart/mixed; boundary=\"abbc761f78ff4d7cb7573b5a23f96ef0\"",
+            ),
+        );
+        (bytes, headers)
+    }
+    fn create_simple_request_with_1_files_array_header() -> (Bytes, HeaderMap) {
+        let bytes = Bytes::from(
+            "testasdadsad\r\n\
+             --abbc761f78ff4d7cb7573b5a23f96ef0\r\n\
+             Content-Disposition: form-data; name=\"files_param[]\"; filename=\"fn.txt\"\r\n\
+             Content-Type: text/plain; charset=utf-8\r\n\r\n\
+             test\r\n\
+             --abbc761f78ff4d7cb7573b5a23f96ef0--\r\n",
+        );
+        let mut headers = HeaderMap::new();
+        headers.insert(
+            header::CONTENT_TYPE,
+            header::HeaderValue::from_static(
+                "multipart/mixed; boundary=\"abbc761f78ff4d7cb7573b5a23f96ef0\"",
+            ),
+        );
+        (bytes, headers)
+    }
+    fn create_simple_request_with_3_files_array_with_name_without_hooks_header() -> (Bytes, HeaderMap) {
+        let bytes = Bytes::from(
+            "testasdadsad\r\n\
+             --abbc761f78ff4d7cb7573b5a23f96ef0\r\n\
+             Content-Disposition: form-data; name=\"files_param\"; filename=\"fn.txt\"\r\n\
+             Content-Type: text/plain; charset=utf-8\r\n\r\n\
+             test\r\n\
+             --abbc761f78ff4d7cb7573b5a23f96ef0\r\n\
+             Content-Disposition: form-data; name=\"files_param\"; filename=\"fn2txt\"\r\n\
+             Content-Type: text/plain; charset=utf-8\r\n\r\n\
+             test\r\n\
+             --abbc761f78ff4d7cb7573b5a23f96ef0\r\n\
+             Content-Disposition: form-data; name=\"files_param\"; filename=\"fn3.txt\"\r\n\
+             Content-Type: text/plain; charset=utf-8\r\n\r\n\
+             test\r\n\
+             --abbc761f78ff4d7cb7573b5a23f96ef0--\r\n",
+        );
+        let mut headers = HeaderMap::new();
+        headers.insert(
+            header::CONTENT_TYPE,
+            header::HeaderValue::from_static(
+                "multipart/mixed; boundary=\"abbc761f78ff4d7cb7573b5a23f96ef0\"",
+            ),
+        );
+        (bytes, headers)
+    }
     fn create_simple_request_with_header_empty_file() -> (Bytes, HeaderMap) {
         let bytes = Bytes::from(
             "testasdadsad\r\n\
@@ -251,10 +408,10 @@ mod tests {
 
         sender.send(Ok(bytes)).unwrap();
 
-        let multipart = Multipart::new(&headers, payload);
+        let actix_multipart = actix_multipart::Multipart::new(&headers, payload);
 
-        match extract_multipart::<Test>(multipart).await {
-            Ok(data) => assert_eq!(data.file_param.size, 4),
+        match extract_multipart::<Test>(actix_multipart).await {
+            Ok(data) => assert_eq!(data.file_param.len(), 4),
             Err(_) => panic!("Failed to parse multipart into structure")
         }
     }
@@ -276,14 +433,37 @@ mod tests {
 
         sender.send(Ok(bytes)).unwrap();
 
-        let multipart = Multipart::new(&headers, payload);
+        let actix_multipart = actix_multipart::Multipart::new(&headers, payload);
 
-        match extract_multipart::<Test>(multipart).await {
+        match extract_multipart::<Test>(actix_multipart).await {
             Ok(_) => panic!("Types not matching, but parsing was a success. It should have return an Err(_)"),
             Err(_) => panic!("Failed to parse multipart into structure")
         }
     }
 
+    #[allow(dead_code)]
+    #[actix_rt::test]
+    async fn testing_primitive_type_array() {
+        #[derive(Deserialize)]
+        struct Test {
+            string_param: Vec<String>,
+            i32_param: Vec<i32>,
+            u32_param: Vec<u32>,
+        }
+
+        let (sender, payload) = create_stream();
+        let (bytes, headers) = create_simple_request_with_array_header();
+
+        sender.send(Ok(bytes)).unwrap();
+
+        let actix_multipart = actix_multipart::Multipart::new(&headers, payload);
+
+        match extract_multipart::<Test>(actix_multipart).await {
+            Ok(data) => assert_eq!(data.string_param.len(), 3),
+            Err(_) => panic!("Failed to parse multipart into structure")
+        }
+    }
+
     #[allow(dead_code)]
     #[actix_rt::test]
     async fn test_empty_file_ignored() {
@@ -300,9 +480,9 @@ mod tests {
 
         sender.send(Ok(bytes)).unwrap();
 
-        let multipart = Multipart::new(&headers, payload);
+        let actix_multipart = actix_multipart::Multipart::new(&headers, payload);
 
-        match extract_multipart::<Test>(multipart).await {
+        match extract_multipart::<Test>(actix_multipart).await {
             Ok(data) => assert_eq!(data.file_param.is_none(), true),
             Err(_) => panic!("Failed to parse multipart into structure")
         }
@@ -324,9 +504,9 @@ mod tests {
 
         sender.send(Ok(bytes)).unwrap();
 
-        let multipart = Multipart::new(&headers, payload);
+        let actix_multipart = actix_multipart::Multipart::new(&headers, payload);
 
-        match extract_multipart::<Test>(multipart).await {
+        match extract_multipart::<Test>(actix_multipart).await {
             Ok(data) => assert_eq!(data.file_param.is_some(), true),
             Err(_) => panic!("Failed to parse multipart into structure")
         }
@@ -348,14 +528,74 @@ mod tests {
 
         sender.send(Ok(bytes)).unwrap();
 
-        let multipart = Multipart::new(&headers, payload);
+        let actix_multipart = actix_multipart::Multipart::new(&headers, payload);
 
-        match extract_multipart::<Test>(multipart).await {
+        match extract_multipart::<Test>(actix_multipart).await {
             Ok(data) => assert_eq!(data.file_param.is_none(), true),
             Err(_) => panic!("Failed to parse multipart into structure")
         }
     }
 
+    #[actix_rt::test]
+    async fn test_multiple_files_param_with_3_files() {
+        #[derive(Deserialize)]
+        struct Test {
+            files_param: Vec<File>,
+        }
+
+        let (sender, payload) = create_stream();
+        let (bytes, headers) = create_simple_request_with_3_files_array_header();
+
+        sender.send(Ok(bytes)).unwrap();
+
+        let actix_multipart = actix_multipart::Multipart::new(&headers, payload);
+
+        match extract_multipart::<Test>(actix_multipart).await {
+            Ok(data) => assert_eq!((data.files_param.len() == 3), true),
+            Err(_) => panic!("Failed to parse multipart into structure")
+        }
+    }
+    #[actix_rt::test]
+    async fn test_multiple_files_param_with_1_file() {
+        #[derive(Deserialize)]
+        struct Test {
+            files_param: Vec<File>,
+        }
+
+        let (sender, payload) = create_stream();
+        let (bytes, headers) = create_simple_request_with_1_files_array_header();
+
+        sender.send(Ok(bytes)).unwrap();
+
+        let actix_multipart = actix_multipart::Multipart::new(&headers, payload);
+
+        match extract_multipart::<Test>(actix_multipart).await {
+            Ok(data) => assert_eq!((data.files_param.len() == 1), true),
+            Err(_) => panic!("Failed to parse multipart into structure")
+        }
+    }
+
+    #[actix_rt::test]
+    #[should_panic(expected = "When uploading multiple files with one field, the field name need to have hooks [] at the end")]
+    async fn test_multiple_files_param_with_3_file_with_name_without_hooks() {
+        #[derive(Deserialize)]
+        struct Test {
+            files_param: Vec<File>,
+        }
+
+        let (sender, payload) = create_stream();
+        let (bytes, headers) = create_simple_request_with_3_files_array_with_name_without_hooks_header();
+
+        sender.send(Ok(bytes)).unwrap();
+
+        let actix_multipart = actix_multipart::Multipart::new(&headers, payload);
+
+        match extract_multipart::<Test>(actix_multipart).await {
+            Ok(data) => assert_eq!((data.files_param.len() == 3), true),
+            Err(_) => panic!("When uploading multiple files with one field, the field name need to have hooks [] at the end")
+        }
+    }
+
     #[actix_rt::test]
     async fn test_value_in_good_param() {
         #[derive(Deserialize)]
@@ -369,11 +609,11 @@ mod tests {
 
         sender.send(Ok(bytes)).unwrap();
 
-        let multipart = Multipart::new(&headers, payload);
+        let actix_multipart = actix_multipart::Multipart::new(&headers, payload);
 
-        match extract_multipart::<Test>(multipart).await {
+        match extract_multipart::<Test>(actix_multipart).await {
             Ok(data) => assert_eq!(if data.param1 == 56 && data.param2 == 24 { true } else { false }, true),
             Err(_) => panic!("Failed to parse multipart into structure")
         }
     }
-}
+}
\ No newline at end of file
